// Generated by CoffeeScript 1.12.6
(function() {
  var MessageHandler, NOT_FOUND_RESPONSE, _, glob, http, path,
    bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };

  _ = require('lodash');

  glob = require('glob');

  path = require('path');

  http = require('http');

  NOT_FOUND_RESPONSE = {
    metadata: {
      code: 404,
      status: http.STATUS_CODES[404]
    }
  };

  MessageHandler = (function() {
    function MessageHandler(arg) {
      this.connector = arg.connector, this.connectorPath = arg.connectorPath, this.defaultJobType = arg.defaultJobType, this.logger = arg.logger;
      this._getJobs = bind(this._getJobs, this);
      this.onMessage = bind(this.onMessage, this);
      if (this.connectorPath == null) {
        throw new Error('MessageHandler requires connectorPath');
      }
      if (this.logger == null) {
        throw new Error('MessageHandler requires logger');
      }
      this.jobs = this._getJobs();
    }

    MessageHandler.prototype.onMessage = function(arg, callback) {
      var data, job, metadata;
      data = arg.data, metadata = arg.metadata;
      if ((metadata != null ? metadata.code : void 0) != null) {
        return callback();
      }
      job = this.jobs[metadata != null ? metadata.jobType : void 0];
      if (this.defaultJobType != null) {
        if (job == null) {
          job = this.jobs[this.defaultJobType];
        }
      }
      if (job == null) {
        return callback(null, NOT_FOUND_RESPONSE);
      }
      this.logger.debug({
        job: job
      }, 'MessageHandler.onMessage');
      return job.action({
        connector: this.connector
      }, {
        data: data,
        metadata: metadata
      }, (function(_this) {
        return function(error, response) {
          if (error != null) {
            return callback(error);
          }
          return callback(null, _.pick(response, 'data', 'metadata'));
        };
      })(this));
    };

    MessageHandler.prototype._getJobs = function() {
      var dirnames, jobs;
      dirnames = glob.sync(path.join(this.connectorPath, 'jobs', '/*/'));
      jobs = {};
      _.each(dirnames, (function(_this) {
        return function(dir) {
          var error, key;
          key = _.upperFirst(_.camelCase(path.basename(dir)));
          try {
            return jobs[key] = require(dir);
          } catch (error1) {
            error = error1;
            _this.logger.debug(error);
            return console.error(error.stack);
          }
        };
      })(this));
      return jobs;
    };

    return MessageHandler;

  })();

  module.exports = MessageHandler;

}).call(this);

//# sourceMappingURL=message-handler.js.map
