// Generated by CoffeeScript 1.12.6
(function() {
  var EXPIRATION_TIMEOUT, MeshbluHttp, MeshbluSocketIO, StatusDevice, UPDATE_INTERVAL, _, async, moment,
    bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };

  async = require('async');

  _ = require('lodash');

  MeshbluSocketIO = require('meshblu');

  MeshbluHttp = require('meshblu-http');

  moment = require('moment');

  UPDATE_INTERVAL = 60 * 1000;

  EXPIRATION_TIMEOUT = 120 * 1000;

  StatusDevice = (function() {
    function StatusDevice(arg) {
      var device, meshblu;
      this.meshbluConfig = arg.meshbluConfig, meshblu = arg.meshblu, device = arg.device, this.checkOnline = arg.checkOnline, this.logger = arg.logger;
      this._updateStatusRef = bind(this._updateStatusRef, this);
      this._updateOnlineUntilToNow = bind(this._updateOnlineUntilToNow, this);
      this._updateOnlineUntil = bind(this._updateOnlineUntil, this);
      this.logError = bind(this.logError, this);
      this._update = bind(this._update, this);
      this._setup = bind(this._setup, this);
      this._findOrCreate = bind(this._findOrCreate, this);
      this._copyDiscoverWhitelist = bind(this._copyDiscoverWhitelist, this);
      this.start = bind(this.start, this);
      this._generateToken = bind(this._generateToken, this);
      this._connect = bind(this._connect, this);
      this._create = bind(this._create, this);
      this.close = bind(this.close, this);
      if (this.logger == null) {
        throw new Error('StatusDevice requires logger');
      }
      this.connectorDevice = device;
      this.connectorMeshblu = meshblu;
      this.tag = "connector-" + this.meshbluConfig.uuid + "-status-device";
      this.update = _.throttle(this._update, 1000, {
        leading: true,
        trailing: false
      });
    }

    StatusDevice.prototype.close = function(callback) {
      if (this._updateOnlineUntilInterval != null) {
        clearInterval(this._updateOnlineUntilInterval);
      }
      return this._updateOnlineUntilToNow((function(_this) {
        return function(error) {
          if (error != null) {
            return callback(error);
          }
          return _this.statusMeshblu.close(function(error) {
            if (error != null) {
              return callback(error);
            }
            return callback();
          });
        };
      })(this));
    };

    StatusDevice.prototype._create = function(callback) {
      var device, uuid;
      uuid = this.meshbluConfig.uuid;
      device = {
        type: 'connector-status-device',
        owner: uuid,
        discoverWhitelist: [uuid, this.connectorDevice.owner],
        configureWhitelist: [uuid, this.connectorDevice.owner],
        sendWhitelist: [uuid, this.connectorDevice.owner],
        receiveWhitelist: [uuid, this.connectorDevice.owner],
        status: {}
      };
      return this.connectorMeshblu.register(device, (function(_this) {
        return function(device) {
          if (device == null) {
            device = {};
          }
          if (device.error != null) {
            return callback(device.error);
          }
          _this.device = device;
          return _this._connect(callback);
        };
      })(this));
    };

    StatusDevice.prototype._connect = function(_callback) {
      var callback, meshbluConfig;
      callback = _.once(_callback);
      meshbluConfig = _.cloneDeep(this.meshbluConfig);
      meshbluConfig.uuid = this.device.uuid;
      meshbluConfig.token = this.device.token;
      this.statusMeshblu = new MeshbluSocketIO(meshbluConfig);
      this.statusMeshbluHttp = new MeshbluHttp(meshbluConfig);
      this.statusMeshblu.once('ready', (function(_this) {
        return function() {
          _this.logger.debug('status device is ready');
          return callback();
        };
      })(this));
      this.statusMeshblu.connect();
      this.statusMeshblu.on('error', (function(_this) {
        return function(error) {
          _this.logger.error(error, 'status device error');
          return callback(error);
        };
      })(this));
      this.statusMeshblu.on('notReady', (function(_this) {
        return function(error) {
          _this.logger.error(error, 'status device notReady');
          return callback(error);
        };
      })(this));
      return this.statusMeshblu.on('message', (function(_this) {
        return function(message) {
          if (message == null) {
            message = {};
          }
          if (message.topic !== 'ping') {
            return;
          }
          _this.logger.debug(message, 'on message');
          return _this.checkOnline(function(error, response) {
            return _this.update({
              error: error,
              response: response
            });
          });
        };
      })(this));
    };

    StatusDevice.prototype._generateToken = function(uuid, callback) {
      return this.connectorMeshblu.revokeTokenByQuery({
        uuid: uuid,
        tag: this.tag
      }, (function(_this) {
        return function(response) {
          if (response == null) {
            response = {};
          }
          if (response.error != null) {
            return callback(response.error);
          }
          return _this.connectorMeshblu.generateAndStoreToken({
            uuid: uuid,
            tag: _this.tag
          }, function(device) {
            if (device == null) {
              device = {};
            }
            if (device.error != null) {
              return callback(device.error);
            }
            _this.device = device;
            return _this._connect(callback);
          });
        };
      })(this));
    };

    StatusDevice.prototype.start = function(callback) {
      return this._setup((function(_this) {
        return function(error) {
          if (error != null) {
            return callback(error);
          }
          _this._updateOnlineUntilInterval = setInterval(_this._updateOnlineUntil, UPDATE_INTERVAL);
          _this._updateOnlineUntil();
          return callback();
        };
      })(this));
    };

    StatusDevice.prototype._copyDiscoverWhitelist = function(callback) {
      var newDiscoverWhitelist, statusDeviceWhitelist;
      newDiscoverWhitelist = _.map(_.get(this.connectorDevice, 'meshblu.whitelists.discover.view', []), 'uuid');
      statusDeviceWhitelist = _.union(this.connectorDevice.discoverWhitelist, newDiscoverWhitelist, [this.connectorDevice.uuid]);
      return this.statusMeshblu.update({
        uuid: this.device.uuid,
        discoverWhitelist: statusDeviceWhitelist
      }, (function(_this) {
        return function() {
          return callback();
        };
      })(this));
    };

    StatusDevice.prototype._findOrCreate = function(callback) {
      if (this.connectorDevice.statusDevice != null) {
        return this._generateToken(this.connectorDevice.statusDevice, callback);
      }
      return this._create(callback);
    };

    StatusDevice.prototype._setup = function(callback) {
      return async.series([this._findOrCreate, this._updateStatusRef, this._copyDiscoverWhitelist], callback);
    };

    StatusDevice.prototype._update = function(arg, callback) {
      var date, error, message, response, uuid;
      response = arg.response, error = arg.error;
      if (callback == null) {
        callback = function() {};
      }
      date = Date.now();
      message = {
        devices: ['*'],
        topic: 'pong',
        date: date,
        response: response,
        error: error
      };
      this.statusMeshblu.message(message);
      uuid = this.device.uuid;
      return this.statusMeshblu.update({
        uuid: uuid,
        lastPong: {
          response: response,
          error: error,
          date: date
        }
      }, (function(_this) {
        return function(error) {
          if (((error != null ? error.uuid : void 0) != null) && (error != null ? error.status : void 0) < 399) {
            return callback(null);
          }
          if (error != null) {
            return callback(error);
          }
          return callback(null);
        };
      })(this));
    };

    StatusDevice.prototype.logError = function(arg, callback) {
      var error, ref, response, update;
      response = arg.response, error = arg.error;
      if (callback == null) {
        callback = function() {};
      }
      this.logger.debug('log error on status device', {
        error: error,
        response: response
      });
      update = {
        $push: {
          errors: {
            $each: [
              {
                date: moment.utc().format(),
                code: (ref = error.code) != null ? ref : 500,
                message: error.message
              }
            ],
            $slice: -99
          }
        }
      };
      return this.statusMeshbluHttp.updateDangerously(this.device.uuid, update, (function(_this) {
        return function(newError) {
          if (newError != null) {
            _this.logger.error(newError.stack);
          }
          return _this.update({
            response: response,
            error: error
          }, function(updateError) {
            if (updateError != null) {
              return _this.logger.error('error updating status device', updateError);
            }
          });
        };
      })(this));
    };

    StatusDevice.prototype._updateOnlineUntil = function() {
      var onlineUntil, uuid;
      uuid = this.device.uuid;
      onlineUntil = moment().utc().add(EXPIRATION_TIMEOUT, 'ms');
      return this.statusMeshblu.update({
        uuid: uuid,
        'status.onlineUntil': onlineUntil
      }, (function(_this) {
        return function() {};
      })(this));
    };

    StatusDevice.prototype._updateOnlineUntilToNow = function(callback) {
      var update;
      update = {
        uuid: this.device.uuid,
        'status.onlineUntil': moment()
      };
      return this.statusMeshblu.update(update, (function(_this) {
        return function() {
          return callback();
        };
      })(this));
    };

    StatusDevice.prototype._updateStatusRef = function(callback) {
      var update;
      update = {
        uuid: this.meshbluConfig.uuid,
        statusDevice: this.device.uuid,
        status: {
          $ref: "meshbludevice://" + this.device.uuid + "/#/status"
        }
      };
      return this.connectorMeshblu.update(update, (function(_this) {
        return function(response) {
          if (response == null) {
            response = {};
          }
          if (response.error != null) {
            return callback(response.error);
          }
          return callback();
        };
      })(this));
    };

    return StatusDevice;

  })();

  module.exports = StatusDevice;

}).call(this);

//# sourceMappingURL=status-device.js.map
